/*
 * Copyright 2009, Haiku, Inc. All Rights Reserved.
 * Distributed under the terms of the MIT License.
 */
#ifndef _IEEE80211_DRIVER_H
#define _IEEE80211_DRIVER_H

/*! Wireless IEEE80211 driver interface */

/* Currently trying to base this on FreeBSD's ieee80211_ioctl.h,
 * which uses a 
 * struct ieee80211req for almost all ioctls (except for stats which
 * uses a simpler struct that is not part of ieee80211. 
 * 
 * Btw do we want to use IEEE80211 or wireless as name?
 * */

#include <Drivers.h>


/* ioctl() opcodes a driver should support */
/* This is a raw list of ioctl 'types' from FreeBSD, made ioctls.
 * This list may/should be compacted as I understand more about
 * every ioctl's function.
enum {
	IEEE80211_IOC_SSID /* TODO: Set this to something B_DEVICE_OP_CODES_END? or a ETHER_OP_CODES_END? */,
	IEEE80211_IOC_NUMSSIDS,
	IEEE80211_IOC_WEP,
	IEEE80211_IOC_WEPKEY,
	IEEE80211_IOC_NUMWEPKEYS,
	IEEE80211_IOC_WEPTXKEY,
	IEEE80211_IOC_AUTHMODE,
	IEEE80211_IOC_STATIONNAME,
	IEEE80211_IOC_CHANNEL,
	IEEE80211_IOC_POWERSAVE,
	IEEE80211_IOC_POWERSAVESLEEP,
	IEEE80211_IOC_RTSTHRESHOLD,
	IEEE80211_IOC_PROTMODE,
	
	IEEE80211_IOC_TXPOWER,					/* global tx power limit */
	IEEE80211_IOC_BSSID,
	IEEE80211_IOC_ROAMING,					/* roaming mode */
	IEEE80211_IOC_PRIVACY,					/* privacy invoked */
	IEEE80211_IOC_DROPUNENCRYPTED,			/* discard unencrypted frames */
	IEEE80211_IOC_WPAKEY,
	IEEE80211_IOC_DELKEY,
	IEEE80211_IOC_MLME,
	IEEE80211_IOC_COUNTERMEASURES,			/* WPA/TKIP countermeasures */
	IEEE80211_IOC_WPA,						/* WPA mode (0,1,2) */
	IEEE80211_IOC_CHANLIST,					/* channel list */
	IEEE80211_IOC_WME,						/* WME mode (on, off) */
	IEEE80211_IOC_HIDESSID,					/* hide SSID mode (on, off) */
	IEEE80211_IOC_APBRIDGE,					/* AP inter-sta bridging */
	IEEE80211_IOC_WPAIE,					/* WPA information element */
	IEEE80211_IOC_STA_STATS,				/* per-station statistics */
	IEEE80211_IOC_MACCMD,					/* MAC ACL operation */
	IEEE80211_IOC_CHANINFO,					/* channel info list */
	IEEE80211_IOC_TXPOWMAX,					/* max tx power for channel */
	IEEE80211_IOC_STA_TXPOW,				/* per-station tx power limit */
	IEEE80211_IOC_WME_CWMIN,				/* WME: ECWmin */
	IEEE80211_IOC_WME_CWMAX,				/* WME: ECWmax */
	IEEE80211_IOC_WME_AIFS,					/* WME: AIFSN */
	IEEE80211_IOC_WME_TXOPLIMIT,			/* WME: txops limit */
	IEEE80211_IOC_WME_ACM,					/* WME: ACM (bss only) */
	IEEE80211_IOC_WME_ACKPOLICY,			/* WME: ACK policy (!bss only)*/
	IEEE80211_IOC_DTIM_PERIOD,				/* DTIM period (beacons) */
	IEEE80211_IOC_BEACON_INTERVAL,			/* beacon interval (ms) */
	IEEE80211_IOC_ADDMAC,					/* add sta to MAC ACL table */
	IEEE80211_IOC_DELMAC,					/* del sta from MAC ACL table */
	IEEE80211_IOC_PUREG,					/* pure 11g (no 11b stations) */
	IEEE80211_IOC_FF,						/* ATH fast frames (on, off) */
	IEEE80211_IOC_TURBOP,					/* ATH turbo' (on, off) */
	IEEE80211_IOC_BGSCAN,					/* bg scanning (on, off) */
	IEEE80211_IOC_BGSCAN_IDLE,				/* bg scan idle threshold */
	IEEE80211_IOC_BGSCAN_INTERVAL,			/* bg scan interval */
	IEEE80211_IOC_SCANVALID,				/* scan cache valid threshold */
	IEEE80211_IOC_FRAGTHRESHOLD,			/* tx fragmentation threshold */
	IEEE80211_IOC_BURST,					/* packet bursting */
	IEEE80211_IOC_SCAN_RESULTS,				/* get scan results */
	IEEE80211_IOC_BMISSTHRESHOLD,			/* beacon miss threshold */
	IEEE80211_IOC_STA_INFO,					/* station/neighbor info */
	IEEE80211_IOC_WPAIE2,					/* WPA+RSN info elements */
	IEEE80211_IOC_CURCHAN,					/* current channel */
	IEEE80211_IOC_SHORTGI,					/* 802.11n half GI */
	IEEE80211_IOC_AMPDU,					/* 802.11n A-MPDU (on, off) */
	IEEE80211_IOC_AMPDU_LIMIT,				/* A-MPDU length limit */
	IEEE80211_IOC_AMPDU_DENSITY,			/* A-MPDU density */
	IEEE80211_IOC_AMSDU,					/* 802.11n A-MSDU (on, off) */
	IEEE80211_IOC_AMSDU_LIMIT,				/* A-MSDU length limit */
	IEEE80211_IOC_PUREN,					/* pure 11n (no legacy sta's) */
	IEEE80211_IOC_DOTH,						/* 802.11h (on, off) */
	IEEE80211_IOC_HTCOMPAT,					/* support pre-D1.10 HT ie's */
	IEEE80211_IOC_DWDS,						/* DWDS/4-address handling */
	IEEE80211_IOC_INACTIVITY,				/* sta inactivity handling */
	IEEE80211_IOC_APPIE,					/* application IE's */
	IEEE80211_IOC_WPS,						/* WPS operation */
	IEEE80211_IOC_TSN,						/* TSN operation */
	IEEE80211_IOC_DEVCAPS,					/* driver+device capabilities */
	IEEE80211_IOC_CHANSWITCH,				/* start 11h channel switch */
	IEEE80211_IOC_DFS,						/* DFS (on, off) */
	IEEE80211_IOC_DOTD,						/* 802.11d (on, off) */
	IEEE80211_IOC_HTPROTMODE,				/* HT protection (off, rts) */
	IEEE80211_IOC_SCAN_REQ,					/* scan w/ specified params */
	IEEE80211_IOC_SCAN_CANCEL,				/* cancel ongoing scan */
	IEEE80211_IOC_HTCONF,					/* HT config (off, HT20, HT40)*/
	IEEE80211_IOC_REGDOMAIN,				/* regulatory domain info */
	IEEE80211_IOC_ROAM,						/* roaming params en masse */
	IEEE80211_IOC_TXPARAMS,					/* tx parameters */
	IEEE80211_IOC_STA_VLAN,					/* per-station vlan tag */
	IEEE80211_IOC_SMPS,						/* MIMO power save */
	IEEE80211_IOC_RIFS,						/* RIFS config (on, off) */

	IEEE80211_IOC_TDMA_SLOT,				/* TDMA: assigned slot */
	IEEE80211_IOC_TDMA_SLOTCNT,				/* TDMA: slots in bss */
	IEEE80211_IOC_TDMA_SLOTLEN,				/* TDMA: slot length (usecs) */
	IEEE80211_IOC_TDMA_BINTERVAL,			/* TDMA: beacon intvl (slots) */	
};

/* I see two kind of requests atm, one for reading/writing simple 16bit value and
 * one for reading/writing complex data. I need more knowledge of individual reqs.

/* values for IEEE80211_IOC_WEP
#define 	IEEE80211_WEP_NOSUP	-1
#define 	IEEE80211_WEP_OFF	0
#define 	IEEE80211_WEP_ON	1
#define 	IEEE80211_WEP_MIXED	2
*/

/* Values for IEEE80211_IOC_POWERSAVE
#define 	IEEE80211_POWERSAVE_NOSUP	-1
#define 	IEEE80211_POWERSAVE_OFF		0
#define 	IEEE80211_POWERSAVE_CAM		1
#define 	IEEE80211_POWERSAVE_PSP		2
#define 	IEEE80211_POWERSAVE_PSP_CAM	3
#define 	IEEE80211_POWERSAVE_ON		IEEE80211_POWERSAVE_CAM
*/

/* Values for IEEE80211_IOC_PROTMODE
#define 	IEEE80211_PROTMODE_OFF		0
#define 	IEEE80211_PROTMODE_CTS		1
#define 	IEEE80211_PROTMODE_RTSCTS	2
*/

/* Below is FreeBSD's struct for communication.
 * AFAICT they send in the name of the interface and the ioctl type
 * in i_type using one ioctl SIOCG80211. 
 * Not sure we want to replicate this but switch to proper ioctls for all
 * types. Also I think we can ditch the name in the request. 
 * i_val - pass a value in or out of ioctl.
 * i_len - length of extra data
 * i_data - pass data in or out of ioctl.
 * 
 */  
#ifdef FREEBSD_CODE_FOR_REF_ONLY
/* the first member must be matched with struct ifreq */
struct ieee80211req {
	char		i_name[IFNAMSIZ];	/* if_name, e.g. "wi0" */
	uint16_t	i_type;			/* req type */
	int16_t		i_val;			/* Index or simple value */
	int16_t		i_len;			/* Index or simple value */
	void		*i_data;		/* Extra data */
};
#endif





#endif	/* _IEEE80211_DRIVER_H */
